/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: 
Contains the implementation of functions described in the RSecureSubSession class.
*/




/**
 @file
*/

#include "secureclientandserver.h"
#include "secureclient.h"

/**
Creates a sub session and returns a handle to it.
@param aSession The session handle.
@return KErrNone if successful, otherwise one of the system-wide error codes.
*/
TInt RSecureSubSession::Open(RSecureSession &aSession)
	{
	return CreateSubSession(aSession,ESecureServerCreateSubSession);
	}

/**
Closes the sub session.
*/
void RSecureSubSession::Close()
	{
	RSubSessionBase::CloseSubSession(ESecureServerCloseSubSession);
	}

/**
Sets the counter value from a string.
@param aString The descriptor containing the counter value.
@return KErrNone if successful, otherwise one of the system-wide error codes.
*/
TInt RSecureSubSession::SetFromString(const TDesC& aString)
	{
	TIpcArgs args(&aString);
	return SendReceive(ESecureServerInitSubSession, args);
	}

/**
Increases the counter value by one.
*/
void RSecureSubSession::Increase()
	{
	if (SubSessionHandle())
		{
		SendReceive(ESecureServerIncrease);
		}
	}

/**
Increases the counter value by the supplied value.
@param aInt The increase in the counter value.
*/
void RSecureSubSession::IncreaseByL(TInt aInt)
	{
	if (SubSessionHandle())
		{
		TIpcArgs args(aInt);
		User::LeaveIfError(SendReceive(ESecureServerIncreaseBy, args));
		}
	}

/**
Decreases the counter value by one.
*/
void RSecureSubSession::Decrease()
	{
	if (SubSessionHandle())
		{
		SendReceive(ESecureServerDecrease);
		}
	}

/**
Decreases the counter value by the supplied value.
@param aInt The decrease in the counter value.
*/
void RSecureSubSession::DecreaseByL(TInt aInt)
	{
	if (SubSessionHandle())
		{
		TIpcArgs args(aInt);
		User::LeaveIfError(SendReceive(ESecureServerDecreaseBy, args));
		}
	}

/**
Resets the counter to zero.
*/
void RSecureSubSession::Reset()
	{
	if (SubSessionHandle())
		{
		SendReceive(ESecureServerReset);
		}
	}

/**
Gets the value of the counter.
@return The counter value.
*/
TInt RSecureSubSession::CounterValueL()
	{
	TInt res = KErrNotFound;
	TPckgBuf<TInt> pckg;

	if (SubSessionHandle())
		{
		TIpcArgs args(&pckg);
		User::LeaveIfError(SendReceive(ESecureServerValue, args));
		res = pckg();
		}
	return res;
	}

/**
Saves the counter value to the file counter.dat.
@see CSecureServerSubSession::SaveCounterValueL()
*/
void RSecureSubSession::SaveCounterL()
	{
	if (SubSessionHandle())
		{
		User::LeaveIfError(SendReceive(ESecureServerSaveCounter));
		}
	}

/**
Sets the counter to the value read from the file counter.dat.
@see CSecureServerSubSession::SetCounterValueFromFileL()
*/
void RSecureSubSession::SetCounterFromFileL()
	{
	if (SubSessionHandle())
		{
		User::LeaveIfError(SendReceive(ESecureServerSetCounterFromFile));
		}
	}
